﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.DataPreprocessing {
  public class SearchLogic : ISearchLogic {
    private readonly ITransactionalPreprocessingData preprocessingData;

    private Dictionary<int, IEnumerable<int>> MissingValueIndicies { get; set; }
    private Dictionary<int, IEnumerable> ValuesWithoutNaN { get; set; }

    public SearchLogic(ITransactionalPreprocessingData thePreprocessingData) {
      preprocessingData = thePreprocessingData;

      MissingValueIndicies = new Dictionary<int, IEnumerable<int>>();
      ValuesWithoutNaN = new Dictionary<int, IEnumerable>();

      preprocessingData.Changed += preprocessingData_Changed;
    }

    void preprocessingData_Changed(object sender, DataPreprocessingChangedEventArgs e)
    {
      MissingValueIndicies.Remove(e.Column);
      ValuesWithoutNaN.Remove(e.Column);
    }

    public IDictionary<string, IEnumerable<int>> GetMissingValueIndices() {
      var dic = new Dictionary<string, IEnumerable<int>>();
      foreach (string variableName in preprocessingData.VariableNames) {
        dic.Add(variableName, GetMissingValueIndices(preprocessingData.GetColumnIndex(variableName)));
      }
      return dic;
    }

    public bool IsMissingValue(int columnIndex, int rowIndex) {
      if (preprocessingData.IsType<double>(columnIndex)) {
        return double.IsNaN(preprocessingData.GetCell<double>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<string>(columnIndex)) {
        return string.IsNullOrEmpty(preprocessingData.GetCell<string>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
        return preprocessingData.GetCell<DateTime>(columnIndex, rowIndex).Equals(DateTime.MinValue);
      } else {
        throw new ArgumentException("cell in column " + columnIndex + " and row index " + rowIndex + " contains a non supported type.");
      }
    }

    public IEnumerable<int> GetMissingValueIndices(int columnIndex) {
      if (!MissingValueIndicies.ContainsKey(columnIndex)){        
          if (preprocessingData.IsType<double>(columnIndex)) {
            MissingValueIndicies[columnIndex] = GetMissingValueIndices<double>(columnIndex);
          } else if (preprocessingData.IsType<string>(columnIndex)) {
            MissingValueIndicies[columnIndex] = GetMissingValueIndices<string>(columnIndex);
          } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
            MissingValueIndicies[columnIndex] = GetMissingValueIndices<DateTime>(columnIndex);
          } else {
            throw new ArgumentException("column " + columnIndex + " contains a non supported type.");
          }
      } 

      return MissingValueIndicies[columnIndex];
   }
    private IEnumerable<int> GetMissingValueIndices<T>(int columnIndex) {
      List<int> missingIndices = new List<int>();
     
      for(int row = 0; row < preprocessingData.Rows; ++row) {
        if (IsMissingValue(columnIndex, row)) {
          missingIndices.Add(row);
        }
      }

      return missingIndices;
    }

    public IEnumerable<T> GetValuesWithoutNaN<T>(int columnIndex)
    {
      if (!ValuesWithoutNaN.ContainsKey(columnIndex))
      {
        List<T> values = new List<T>();

        for (int row = 0; row < preprocessingData.Rows; ++row)
        {
          if (!IsMissingValue(columnIndex, row))
          {
            values.Add(preprocessingData.GetCell<T>(columnIndex, row));
          }
        }

        ValuesWithoutNaN[columnIndex] = values;
      }

      return (IEnumerable<T>)ValuesWithoutNaN[columnIndex];
    }
  }
}
